﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Reflection;

namespace howto_reflection_list_properties
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Add some properties.
        private int _MyPrivateProperty;
        private int MyPrivateProperty
        {
            get { return _MyPrivateProperty; }
            set { _MyPrivateProperty = value; }
        }
        public int MyPublicProperty
        {
            get { return 2; }
            set { }
        }
        public static int MyPublicStaticProperty
        {
            get { return 3; }
        }
        protected int MyProtectedProperty
        {
            get { return 4; }
        }
        public virtual int MyPublicVirtualProperty
        {
            set { }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // Make column headers.
            lvwProperties.Columns.Clear();
            lvwProperties.Columns.Add("Property", 10,
                HorizontalAlignment.Left);
            lvwProperties.Columns.Add("Type", 10,
                HorizontalAlignment.Left);
            lvwProperties.Columns.Add("Attributes", 10,
                HorizontalAlignment.Left);
            lvwProperties.Columns.Add("Value", 10,
                HorizontalAlignment.Left);

            // List the properties.
            // Use the class you want to study instead of Form1.
            object property_value;
            PropertyInfo[] property_infos = typeof(Form1).GetProperties(
                BindingFlags.FlattenHierarchy |
                BindingFlags.Instance |
                BindingFlags.NonPublic |
                BindingFlags.Public |
                BindingFlags.Static);
            foreach (PropertyInfo info in property_infos)
            {
                string name = info.Name;
                string attributes = info.PropertyType.Attributes.ToString();
                if (info.CanRead) attributes += " get";
                if (info.CanWrite) attributes += " set";

                string value = "";

                // See if it's an array.
                if (!info.PropertyType.IsArray)
                {
                    // It's not an array.
                    if (info.CanRead) property_value = info.GetValue(this, null);
                    else property_value = "---";

                    if (property_value == null)
                        value = "<null>";
                    else
                        value = property_value.ToString();
                }
                else
                {
                    // It is an array.
                    name += "[]";
                    value = "<array>";
                }

                ListViewMakeRow(lvwProperties, name,
                    info.PropertyType.ToString(),
                    attributes, value);
            }

            // Size the columns to fit the data.
            for (int i = 0; i < lvwProperties.Columns.Count; i++)
                lvwProperties.Columns[i].Width = -2;

            // Size the form.
            int new_wid = 30;
            for (int i = 0; i < lvwProperties.Columns.Count; i++)
                new_wid += lvwProperties.Columns[i].Width;
            this.Size = new Size(new_wid, this.Size.Height);
        }

        // Make a ListView row.
        private void ListViewMakeRow(ListView lvw, String item_title, params string[] subitem_titles)
        {
            // Make the item.
            ListViewItem new_item = lvw.Items.Add(item_title);

            // Make the sub-items.
            for (int i = subitem_titles.GetLowerBound(0); i <= subitem_titles.GetUpperBound(0); i++)
            {
                new_item.SubItems.Add(subitem_titles[i]);
            }
        }
    }
}
